package ru.dbotthepony.kstarbound.client.gl.shader

import org.lwjgl.opengl.GL46.*
import ru.dbotthepony.kstarbound.client.gl.checkForGLError
import java.io.File

class GLShader(
	body: String,
	type: Int
) {
	constructor(body: File, type: Int) : this(body.also { require(it.exists()) { "Shader file does not exists: $body" } }.readText(), type)

	val pointer = glCreateShader(type)
	var unlinked = false
		private set

	init {
		if (body == "") {
			throw IllegalArgumentException("Shader source is empty")
		}

		glShaderSource(pointer, body)
		glCompileShader(pointer)

		val result = intArrayOf(0)
		glGetShaderiv(pointer, GL_COMPILE_STATUS, result)

		if (result[0] == 0) {
			throw ShaderCompilationException(glGetShaderInfoLog(pointer))
		}

		checkForGLError()
	}

	fun unlink(): Boolean {
		if (unlinked)
			return false

		glDeleteShader(pointer)
		checkForGLError()
		return true
	}

	companion object {
		fun vertex(file: File) = GLShader(file, GL_VERTEX_SHADER)
		fun fragment(file: File) = GLShader(file, GL_FRAGMENT_SHADER)

		private fun readInternal(file: String): String {
			return ClassLoader.getSystemClassLoader().getResourceAsStream(file)!!.bufferedReader()
				.let {
					val read = it.readText()
					it.close()
					return@let read
				}
		}

		fun internalVertex(file: String) = GLShader(readInternal(file), GL_VERTEX_SHADER)
		fun internalFragment(file: String) = GLShader(readInternal(file), GL_FRAGMENT_SHADER)
		fun internalGeometry(file: String) = GLShader(readInternal(file), GL_GEOMETRY_SHADER)
	}
}
