package ru.dbotthepony.kstarbound.client.render.entity

import ru.dbotthepony.kstarbound.client.ClientChunk
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.render.entity.ItemRenderer
import ru.dbotthepony.kstarbound.client.render.entity.ProjectileRenderer
import ru.dbotthepony.kstarbound.world.entities.Entity
import ru.dbotthepony.kstarbound.world.entities.ItemEntity
import ru.dbotthepony.kstarbound.world.entities.projectile.Projectile
import ru.dbotthepony.kvector.matrix.Matrix4fStack
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import java.io.Closeable

/**
 * Pseudo Z position for entities, for them to appear behind tile geometry,
 * but in front of background walls geometry
 */
const val Z_LEVEL_ENTITIES = 30000

/**
 * Базовый класс, отвечающий за отрисовку определённого ентити в мире
 *
 * Считается, что процесс отрисовки ограничен лишь одним слоем (т.е. отрисовка происходит в один проход)
 */
open class EntityRenderer(val state: GLStateTracker, val entity: Entity, open var chunk: ClientChunk?) : Closeable {
	open val renderPos: Vector2d get() = entity.position

	open fun render(stack: Matrix4fStack) {

	}

	open fun renderDebug() {
		if (chunk?.world?.client?.settings?.debugCollisions == true) {
			state.quadWireframe(entity.movement.worldAABB)
		}
	}

	open val layer: Int = Z_LEVEL_ENTITIES

	override fun close() {

	}

	companion object {
		fun getRender(state: GLStateTracker, entity: Entity, chunk: ClientChunk? = null): EntityRenderer {
			return when (entity) {
				is Projectile -> ProjectileRenderer(state, entity, chunk)
				is ItemEntity -> ItemRenderer(state, entity, chunk)
				else -> EntityRenderer(state, entity, chunk)
			}
		}
	}
}

